@@ -3,12 +3,14 @@ from .base import InstanceProvider, Instance
 from ..typehints import KT, DT, VT, RT
 from typing import Any, Dict, TypeVar, Generic
 
-IT = TypeVar("IT", bound="Instance[Any, Any, Any, Any]", covariant=True)
+IT = TypeVar("IT", bound="Instance[Any, Any, Any, Any]")
 
 
-class ExternalProvider(InstanceProvider[IT, KT, DT, VT, RT], Generic[IT, KT, DT, VT, RT]):
+class ExternalProvider(
+    InstanceProvider[IT, KT, DT, VT, RT], Generic[IT, KT, DT, VT, RT]
+):
     instance_cache: Dict[KT, IT]
-    
+
     @abstractmethod
     def build_from_external(self, k: KT) -> IT:
         raise NotImplementedError
@@ -28,8 +30,6 @@ class ExternalProvider(InstanceProvider[IT, KT, DT, VT, RT], Generic[IT, KT, DT,
             instance = self.build_from_external(k)
             self.instance_cache[k] = instance
             return instance
-        raise KeyError(f"Instance with key {k} is not present in this provider")
-
-
-
-        
\ No newline at end of file
+        raise KeyError(
+            f"Instance with key {k} is not present in this provider"
+        )
